#include <stdio.h>
#include "parse.h"

static const char *token_type_to_str(TokenType type) {
    switch (type) {
    case TOKEN_TYPE_EOF:
        return "EOF";
    case TOKEN_TYPE_COMMENT:
        return "COMMENT";
    case TOKEN_TYPE_PAREN:
        return "PAREN";
    case TOKEN_TYPE_BRACKET:
        return "BRACKET";
    case TOKEN_TYPE_SYMBOL:
        return "SYMBOL";
    case TOKEN_TYPE_QUOTE:
        return "QUOTE";
    case TOKEN_TYPE_NUMBER:
        return "NUMBER";
    case TOKEN_TYPE_CHAR:
        return "CHAR";
    case TOKEN_TYPE_STRING:
        return "STRING";
    default:
        return "UNKNOWN";
    }
}

int main(int argc, const char **argv) {
    init_lisp();
    LispObject *msg_slot = ref_lisp_object(INTERN_LITERAL("message"));
    jmp_buf jmp;
    int branch = setjmp(jmp);
    if (!branch) {
        LispHashTable *handlers = (void *) call_native(Fmake_hash_table, 2, Vnil, Vnil);
        call_native(Fputhash, 3, handlers, type_error_class, make_lisp_int(1));
        call_native(Fputhash, 3, handlers, class_slot_error_class, make_lisp_int(2));
        StackFramePlace err_place = {
            .file = STRING_FROM_LITERAL(__FILE__),
            .is_native = true,
            .line = __LINE__ - 3,
            .col = 8,
        };
        push_stack((void *) INTERN_LITERAL(__FUNCTION__), &err_place, &jmp, handlers);
        unref_lisp_object(handlers);
        AS_STRING(Vnil);
    } else if (branch == 1) {
        LispString *msg = (void *) call_native(Fslot, 2, current_error, msg_slot);
        printf("Caught %s: %*s\n",
               current_error->class->name->name->text,
               (int) msg->length, msg->text);
        dump_stack(current_error);
    }
    unref_lisp_object(msg_slot);
    pop_stack();
    deinit_lisp();
    return 0;
}
